/*
 * Copyright 2005 Joe Walker
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.directwebremoting.bayeux;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.directwebremoting.Container;
import org.directwebremoting.dwrp.PlainCallMarshaller;
import org.directwebremoting.extend.ConverterManager;
import org.directwebremoting.extend.Remoter;
import org.directwebremoting.impl.ContainerUtil;
import org.directwebremoting.servlet.FileHandler;
import org.directwebremoting.util.MimeConstants;

import dojox.cometd.Bayeux;

/**
 * @author Joe Walker [joe at getahead dot ltd dot uk]
 */
public class DwrBayeuxFilter implements Filter
{
    public void init(FilterConfig filterConfig) throws ServletException
    {
        config=filterConfig;
        initialized=false;
    }
    
    public void destroy()
    {
    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException
    {
        synchronized(this)
        {
            if (!initialized)
            {
                Bayeux bayeux = (Bayeux)config.getServletContext().getAttribute("dojox.cometd.bayeux");

                Remoter remoter=null;
                ConverterManager converterManager=null;
                PlainCallMarshaller plainCallMarshaller=null;
                List<Container> containers = (List<Container>)
                config.getServletContext().getAttribute(ContainerUtil.ATTRIBUTE_CONTAINER_LIST);

                for (Container container: containers)
                {
                    System.err.println("container="+container);
                    System.err.println("beans="+container.getBeanNames());
                    Remoter r = (Remoter)container.getBean("org.directwebremoting.extend.Remoter");
                    if (r!=null)
                    {
                        remoter=r;
                        if (bayeuxHandler!=null)
                            bayeuxHandler.setRemoter(remoter);
                        System.err.println("remoter="+remoter);
                    }

                    ConverterManager c = (ConverterManager)container.getBean("org.directwebremoting.extend.ConverterManager");
                    if (c!=null)
                    {
                        converterManager=c;
                        if (bayeuxHandler!=null)
                            bayeuxHandler.setConverterManager(converterManager);
                        System.err.println("converterManager="+converterManager);
                    }

                    PlainCallMarshaller p = (PlainCallMarshaller)container.getBean("org.directwebremoting.dwrp.PlainCallMarshaller");
                    if (p!=null)
                    {
                        plainCallMarshaller=p;
                        if (bayeuxHandler!=null)
                            bayeuxHandler.setPlainCallMarshaller(plainCallMarshaller);
                        System.err.println("plainCallMarshaller="+plainCallMarshaller);
                    } 
                }

                InputStream engin = this.getClass().getClassLoader().getResourceAsStream("/org/directwebremoting/bayeux/dwrbayeux.js");

                if (engin==null)
                    throw new IllegalStateException("No dwrbayeux.js");
                if (bayeux==null)
                    throw new IllegalStateException("No Bayeux");
                if (remoter==null)
                    throw new IllegalStateException("No Remoter");
                if (converterManager==null)
                    throw new IllegalStateException("No converterManager");
                if (plainCallMarshaller==null)
                    throw new IllegalStateException("No plainCallMarshaller");

                bayeuxHandler = new BayeuxClient(bayeux);
                bayeuxHandler.setRemoter(remoter);
                bayeuxHandler.setConverterManager(converterManager);
                bayeuxHandler.setPlainCallMarshaller(plainCallMarshaller);

                dwrBayeuxJs = new FileHandler("/bayeux/dwrbayeux.js",MimeConstants.MIME_JS,true);

                initialized=true;
            }
        }

        if (((HttpServletRequest)request).getPathInfo().equals("/dwrbayeux.js"))
            dwrBayeuxJs.handle((HttpServletRequest)request, (HttpServletResponse)response);
        else
            chain.doFilter(request,response);

    }

    private BayeuxClient bayeuxHandler;
    private FilterConfig config;
    private boolean initialized;
    private FileHandler dwrBayeuxJs;
    
}
